<?php declare(strict_types=1);

namespace Lubian\AttributeMagic\Infrastructure\Event;

use Invoker\InvokerInterface;
use Lubian\AttributeMagic\Infrastructure\Finder;

use function file_exists;
use function file_get_contents;
use function file_put_contents;
use function serialize;
use function unserialize;

final readonly class DispatcherFactory
{
    public const CACHE_FILE = __DIR__ . '/../../../var/listenerCache';

    public function __construct(
        private Finder $finder,
        private InvokerInterface $invoker
    ) {
    }

    public function getDispatcher(bool $cached = false): Dispatcher
    {
        return new Dispatcher($this->invoker, $this->getListeners($cached));
    }

    /**
     * @return Listener[]
     */
    private function getListeners(bool $cached = false): array
    {
        if ($cached === true && file_exists(self::CACHE_FILE)) {
            $data = file_get_contents(self::CACHE_FILE);
            if ($data === false) {
                return [];
            }

            /** @var Listener[] $listeners */
            $listeners = unserialize($data);
            return $listeners;
        }

        $listeners = $this->finder->getListeners();
        if ($cached === true) {
            file_put_contents(self::CACHE_FILE, serialize($listeners));
        }

        return $listeners;
    }
}
