<?php declare(strict_types=1);

namespace Lubian\AttributeMagic\Infrastructure\WebApp\Route;

use Invoker\InvokerInterface;
use Lubian\AttributeMagic\Infrastructure\Event\AsListener;
use Lubian\AttributeMagic\Infrastructure\WebApp\Request\RequestEvent;
use Symfony\Component\HttpFoundation\Response;

use function ob_get_clean;
use function ob_start;

final readonly class HandlerCaller
{
    public function __construct(
        private InvokerInterface $invoker
    ) {
    }

    #[AsListener(RequestEvent::class, -50)]
    public function callHandler(RequestEvent $event): void
    {
        if ($event->handler === null) {
            $event->stopped = true;
            $event->response = new Response('not resolved', 500);
            return;
        }

        ob_start();
        /** @var mixed $response */
        $response = $this->invoker->call(
            [$event->handler->handlerClass, $event->handler->handlerMethod],
            $event->request->attributes->all()
        );
        $output = (string) ob_get_clean();

        if ($response instanceof Response) {
            $event->response = $response;
            return;
        }

        $event->response = new Response($output, 200);
    }
}
