<?php declare(strict_types=1);

namespace Lubian\AttributeMagic\Infrastructure\WebApp\Route;

use FastRoute\RouteCollector;
use Lubian\AttributeMagic\Infrastructure\Event\AsListener;
use Lubian\AttributeMagic\Infrastructure\Event\Dispatcher;
use Lubian\AttributeMagic\Infrastructure\Route\Handler;
use Lubian\AttributeMagic\Infrastructure\Route\HttpMethod;
use Lubian\AttributeMagic\Infrastructure\WebApp\Request\RequestEvent;
use Symfony\Component\HttpFoundation\Response;

use function FastRoute\cachedDispatcher;

final readonly class HandlerResolver
{
    public function __construct(
        private Dispatcher $dispatcher,
        private bool $cached = false,
    ) {
    }

    #[AsListener(RequestEvent::class, -90)]
    public function resolveHandler(RequestEvent $event): void
    {
        $routesEvent = new CollectRoutes($this->cached);
        $this->dispatcher->dispatch($routesEvent);
        $dispatcher = $this->dispatcher;

        $routeDispatcher = cachedDispatcher(static function (RouteCollector $r) use (
            $dispatcher,
            $routesEvent
        ): void {
            $dispatcher->dispatch($routesEvent);
            foreach ($routesEvent->routes as $h) {
                $r->addRoute($h->method->value, $h->path, [$h->handlerClass, $h->handlerMethod]);
            }
        }, [
            'cacheFile' => __DIR__ . '/../../../../var/route.cache',
            'cacheDisabled' => ! $this->cached,
        ]);

        $routeInfo = $routeDispatcher->dispatch(
            $event->request->getMethod(),
            $event->request->getPathInfo(),
        );

        if ($routeInfo[0] === \FastRoute\Dispatcher::NOT_FOUND) {
            $event->response = new Response('Not Found', 404);
            $event->stopped = true;
            return;
        }

        if ($routeInfo[0] === \FastRoute\Dispatcher::METHOD_NOT_ALLOWED) {
            $event->response = new Response('Not Allowed', 403);
            $event->stopped = true;
            return;
        }

        $event->handler = new Handler(
            HttpMethod::from($event->request->getMethod()),
            $event->request->getPathInfo(),
            ...$routeInfo[1],
        );

        $event->request->attributes->add($routeInfo[2]);
    }
}
